/* Generated automatically by the program `genconstants'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CONSTANTS_H
#define GCC_INSN_CONSTANTS_H

#define SFBOOL_BOOL_A1 5
#define SFBOOL_BOOL_A2 6
#define VSX_TEST_DATA_CLASS_NEG_ZERO 0x4
#define LAST_ALTIVEC_REGNO 95
#define FIRST_FPR_REGNO 32
#define TFIAR_SPR 129
#define SFBOOL_BOOL_D 4
#define FIRST_ALTIVEC_REGNO 64
#define STATIC_CHAIN_REGNUM 11
#define SFBOOL_MFVSR_A 3
#define SFBOOL_MFVSR_D 2
#define TEXASR_SPR 130
#define VSX_TEST_DATA_CLASS_NEG_DENORMAL 0x1
#define CA_REGNO 98
#define SFBOOL_TMP_VSX_DI 12
#define SFBOOL_SHL_A 8
#define CR6_REGNO 106
#define ARG_POINTER_REGNUM 99
#define MAX_CR_REGNO 107
#define TEXASRU_SPR 131
#define VRSAVE_REGNO 108
#define VSX_TEST_DATA_CLASS_NEG_INF 0x10
#define TOC_REGNUM 2
#define VSX_TEST_DATA_CLASS_POS_INF 0x20
#define SFBOOL_MFVSR_A_V4SF 10
#define FIRST_GPR_REGNO 0
#define CR1_REGNO 101
#define LAST_GPR_REGNO 31
#define TFHAR_SPR 128
#define VSX_TEST_DATA_CLASS_NAN 0x40
#define SFBOOL_MTVSR_D 9
#define SFBOOL_TMP_VSX 1
#define CR2_REGNO 102
#define VSX_TEST_DATA_CLASS_POS_ZERO 0x8
#define MAX_HTM_OPERANDS 4
#define MAX_MMA_OPERANDS 7
#define CR3_REGNO 103
#define VSCR_REGNO 109
#define CTR_REGNO 97
#define SFBOOL_MTVSR_D_V4SF 13
#define CR0_REGNO 100
#define SFBOOL_BOOL_A_DI 11
#define CR4_REGNO 104
#define FRAME_POINTER_REGNUM 110
#define CR5_REGNO 105
#define SFBOOL_TMP_GPR 0
#define LAST_FPR_REGNO 63
#define SFBOOL_SHL_D 7
#define CR7_REGNO 107
#define HARD_FRAME_POINTER_REGNUM 31
#define VSX_TEST_DATA_CLASS_POS_DENORMAL 0x2
#define LR_REGNO 96
#define STACK_POINTER_REGNUM 1

enum epilogue_type {
  EPILOGUE_TYPE_NORMAL = 0,
  EPILOGUE_TYPE_SIBCALL = 1,
  EPILOGUE_TYPE_EH_RETURN = 2
};
#define NUM_EPILOGUE_TYPE_VALUES 3
extern const char *const epilogue_type_strings[];

enum unspec {
  UNSPEC_PROBE_STACK = 0,
  UNSPEC_TOCPTR = 1,
  UNSPEC_TOC = 2,
  UNSPEC_TOCSLOT = 3,
  UNSPEC_MOVSI_GOT = 4,
  UNSPEC_FCTIWZ = 5,
  UNSPEC_FRIM = 6,
  UNSPEC_FRIN = 7,
  UNSPEC_FRIP = 8,
  UNSPEC_FRIZ = 9,
  UNSPEC_XSRDPI = 10,
  UNSPEC_LD_MPIC = 11,
  UNSPEC_RELD_MPIC = 12,
  UNSPEC_MPIC_CORRECT = 13,
  UNSPEC_TLSGD = 14,
  UNSPEC_TLSLD = 15,
  UNSPEC_TLS_GET_ADDR = 16,
  UNSPEC_MOVESI_FROM_CR = 17,
  UNSPEC_MOVESI_TO_CR = 18,
  UNSPEC_XVTLSBB = 19,
  UNSPEC_TLSDTPREL = 20,
  UNSPEC_TLSDTPRELHA = 21,
  UNSPEC_TLSDTPRELLO = 22,
  UNSPEC_TLSGOTDTPREL = 23,
  UNSPEC_TLSTPREL = 24,
  UNSPEC_TLSTPRELHA = 25,
  UNSPEC_TLSTPRELLO = 26,
  UNSPEC_TLSGOTTPREL = 27,
  UNSPEC_TLSTLS = 28,
  UNSPEC_TLSTLS_PCREL = 29,
  UNSPEC_FIX_TRUNC_TF = 30,
  UNSPEC_STFIWX = 31,
  UNSPEC_POPCNTB = 32,
  UNSPEC_FRES = 33,
  UNSPEC_SP_SET = 34,
  UNSPEC_SP_TEST = 35,
  UNSPEC_SYNC = 36,
  UNSPEC_LWSYNC = 37,
  UNSPEC_SYNC_OP = 38,
  UNSPEC_ATOMIC = 39,
  UNSPEC_CMPXCHG = 40,
  UNSPEC_XCHG = 41,
  UNSPEC_AND = 42,
  UNSPEC_DLMZB = 43,
  UNSPEC_DLMZB_CR = 44,
  UNSPEC_DLMZB_STRLEN = 45,
  UNSPEC_RSQRT = 46,
  UNSPEC_TOCREL = 47,
  UNSPEC_MACHOPIC_OFFSET = 48,
  UNSPEC_BPERM = 49,
  UNSPEC_PARITY = 50,
  UNSPEC_CMPB = 51,
  UNSPEC_FCTIW = 52,
  UNSPEC_FCTID = 53,
  UNSPEC_LFIWAX = 54,
  UNSPEC_LFIWZX = 55,
  UNSPEC_FCTIWUZ = 56,
  UNSPEC_NOP = 57,
  UNSPEC_GRP_END_NOP = 58,
  UNSPEC_P8V_FMRGOW = 59,
  UNSPEC_P8V_MTVSRWZ = 60,
  UNSPEC_P8V_RELOAD_FROM_GPR = 61,
  UNSPEC_P8V_MTVSRD = 62,
  UNSPEC_P8V_XXPERMDI = 63,
  UNSPEC_P8V_RELOAD_FROM_VSX = 64,
  UNSPEC_ADDG6S = 65,
  UNSPEC_CDTBCD = 66,
  UNSPEC_CBCDTD = 67,
  UNSPEC_DIVE = 68,
  UNSPEC_DIVEU = 69,
  UNSPEC_UNPACK_128BIT = 70,
  UNSPEC_PACK_128BIT = 71,
  UNSPEC_LSQ = 72,
  UNSPEC_FUSION_GPR = 73,
  UNSPEC_STACK_CHECK = 74,
  UNSPEC_CMPRB = 75,
  UNSPEC_CMPRB2 = 76,
  UNSPEC_CMPEQB = 77,
  UNSPEC_ADD_ROUND_TO_ODD = 78,
  UNSPEC_SUB_ROUND_TO_ODD = 79,
  UNSPEC_MUL_ROUND_TO_ODD = 80,
  UNSPEC_DIV_ROUND_TO_ODD = 81,
  UNSPEC_FMA_ROUND_TO_ODD = 82,
  UNSPEC_SQRT_ROUND_TO_ODD = 83,
  UNSPEC_TRUNC_ROUND_TO_ODD = 84,
  UNSPEC_SIGNBIT = 85,
  UNSPEC_SF_FROM_SI = 86,
  UNSPEC_SI_FROM_SF = 87,
  UNSPEC_PLTSEQ = 88,
  UNSPEC_PLT16_HA = 89,
  UNSPEC_CFUGED = 90,
  UNSPEC_CNTLZDM = 91,
  UNSPEC_CNTTZDM = 92,
  UNSPEC_PDEPD = 93,
  UNSPEC_PEXTD = 94,
  UNSPEC_HASHST = 95,
  UNSPEC_HASHCHK = 96,
  UNSPEC_XXSPLTIDP_CONST = 97,
  UNSPEC_XXSPLTIW_CONST = 98,
  UNSPEC_FMAX = 99,
  UNSPEC_FMIN = 100,
  UNSPEC_PREDICATE = 101,
  UNSPEC_REDUC = 102,
  UNSPEC_NEZ_P = 103,
  UNSPEC_VSX_CONCAT = 104,
  UNSPEC_VSX_CVDPSXWS = 105,
  UNSPEC_VSX_CVDPUXWS = 106,
  UNSPEC_VSX_CVSPDP = 107,
  UNSPEC_VSX_CVHPSP = 108,
  UNSPEC_VSX_CVSPDPN = 109,
  UNSPEC_VSX_CVDPSPN = 110,
  UNSPEC_VSX_CVSXWDP = 111,
  UNSPEC_VSX_CVUXWDP = 112,
  UNSPEC_VSX_CVSXDSP = 113,
  UNSPEC_VSX_CVUXDSP = 114,
  UNSPEC_VSX_FLOAT2 = 115,
  UNSPEC_VSX_UNS_FLOAT2 = 116,
  UNSPEC_VSX_FLOATE = 117,
  UNSPEC_VSX_UNS_FLOATE = 118,
  UNSPEC_VSX_FLOATO = 119,
  UNSPEC_VSX_UNS_FLOATO = 120,
  UNSPEC_VSX_TDIV = 121,
  UNSPEC_VSX_TSQRT = 122,
  UNSPEC_VSX_SET = 123,
  UNSPEC_VSX_ROUND_I = 124,
  UNSPEC_VSX_ROUND_IC = 125,
  UNSPEC_VSX_SLDWI = 126,
  UNSPEC_VSX_XXPERM = 127,
  UNSPEC_VSX_XXSPLTW = 128,
  UNSPEC_VSX_XXSPLTD = 129,
  UNSPEC_VSX_DIVSD = 130,
  UNSPEC_VSX_DIVUD = 131,
  UNSPEC_VSX_DIVSQ = 132,
  UNSPEC_VSX_DIVUQ = 133,
  UNSPEC_VSX_DIVESQ = 134,
  UNSPEC_VSX_DIVEUQ = 135,
  UNSPEC_VSX_MODSQ = 136,
  UNSPEC_VSX_MODUQ = 137,
  UNSPEC_VSX_MULSD = 138,
  UNSPEC_VSX_SIGN_EXTEND = 139,
  UNSPEC_VSX_XVCVBF16SPN = 140,
  UNSPEC_VSX_XVCVSPBF16 = 141,
  UNSPEC_VSX_XVCVSPSXDS = 142,
  UNSPEC_VSX_XVCVSPHP = 143,
  UNSPEC_VSX_VSLO = 144,
  UNSPEC_VSX_EXTRACT = 145,
  UNSPEC_VSX_SXEXPDP = 146,
  UNSPEC_VSX_SXSIG = 147,
  UNSPEC_VSX_SIEXPDP = 148,
  UNSPEC_VSX_SIEXPQP = 149,
  UNSPEC_VSX_SCMPEXPDP = 150,
  UNSPEC_VSX_SCMPEXPQP = 151,
  UNSPEC_VSX_STSTDC = 152,
  UNSPEC_VSX_VEXTRACT_FP_FROM_SHORTH = 153,
  UNSPEC_VSX_VEXTRACT_FP_FROM_SHORTL = 154,
  UNSPEC_VSX_VXEXP = 155,
  UNSPEC_VSX_VXSIG = 156,
  UNSPEC_VSX_VIEXP = 157,
  UNSPEC_VSX_VTSTDC = 158,
  UNSPEC_VSX_VSIGNED2 = 159,
  UNSPEC_LXVL = 160,
  UNSPEC_LXVLL = 161,
  UNSPEC_LVSL_REG = 162,
  UNSPEC_LVSR_REG = 163,
  UNSPEC_STXVL = 164,
  UNSPEC_STXVLL = 165,
  UNSPEC_XL_LEN_R = 166,
  UNSPEC_XST_LEN_R = 167,
  UNSPEC_VCLZLSBB = 168,
  UNSPEC_VCTZLSBB = 169,
  UNSPEC_VEXTUBLX = 170,
  UNSPEC_VEXTUHLX = 171,
  UNSPEC_VEXTUWLX = 172,
  UNSPEC_VEXTUBRX = 173,
  UNSPEC_VEXTUHRX = 174,
  UNSPEC_VEXTUWRX = 175,
  UNSPEC_VCMPNEB = 176,
  UNSPEC_VCMPNEZB = 177,
  UNSPEC_VCMPNEH = 178,
  UNSPEC_VCMPNEZH = 179,
  UNSPEC_VCMPNEW = 180,
  UNSPEC_VCMPNEZW = 181,
  UNSPEC_XXEXTRACTUW = 182,
  UNSPEC_XXINSERTW = 183,
  UNSPEC_VSX_FIRST_MATCH_INDEX = 184,
  UNSPEC_VSX_FIRST_MATCH_EOS_INDEX = 185,
  UNSPEC_VSX_FIRST_MISMATCH_INDEX = 186,
  UNSPEC_VSX_FIRST_MISMATCH_EOS_INDEX = 187,
  UNSPEC_XXGENPCV = 188,
  UNSPEC_MTVSBM = 189,
  UNSPEC_EXTENDDITI2 = 190,
  UNSPEC_VCNTMB = 191,
  UNSPEC_VEXPAND = 192,
  UNSPEC_VEXTRACT = 193,
  UNSPEC_EXTRACTL = 194,
  UNSPEC_EXTRACTR = 195,
  UNSPEC_INSERTL = 196,
  UNSPEC_INSERTR = 197,
  UNSPEC_REPLACE_ELT = 198,
  UNSPEC_REPLACE_UN = 199,
  UNSPEC_VDIVES = 200,
  UNSPEC_VDIVEU = 201,
  UNSPEC_VMSUMCUD = 202,
  UNSPEC_XXEVAL = 203,
  UNSPEC_XXSPLTIW = 204,
  UNSPEC_XXSPLTIDP = 205,
  UNSPEC_XXSPLTI32DX = 206,
  UNSPEC_XXBLEND = 207,
  UNSPEC_XXPERMX = 208,
  UNSPEC_VCMPBFP = 209,
  UNSPEC_VMSUMU = 210,
  UNSPEC_VMSUMUDM = 211,
  UNSPEC_VMSUMM = 212,
  UNSPEC_VMSUMSHM = 213,
  UNSPEC_VMSUMUHS = 214,
  UNSPEC_VMSUMSHS = 215,
  UNSPEC_VMHADDSHS = 216,
  UNSPEC_VMHRADDSHS = 217,
  UNSPEC_VADDCUW = 218,
  UNSPEC_VAVGU = 219,
  UNSPEC_VAVGS = 220,
  UNSPEC_VMULEUB = 221,
  UNSPEC_VMULESB = 222,
  UNSPEC_VMULEUH = 223,
  UNSPEC_VMULESH = 224,
  UNSPEC_VMULEUW = 225,
  UNSPEC_VMULESW = 226,
  UNSPEC_VMULEUD = 227,
  UNSPEC_VMULESD = 228,
  UNSPEC_VMULOUB = 229,
  UNSPEC_VMULOSB = 230,
  UNSPEC_VMULOUH = 231,
  UNSPEC_VMULOSH = 232,
  UNSPEC_VMULOUW = 233,
  UNSPEC_VMULOSW = 234,
  UNSPEC_VMULOUD = 235,
  UNSPEC_VMULOSD = 236,
  UNSPEC_VPKPX = 237,
  UNSPEC_VPACK_SIGN_SIGN_SAT = 238,
  UNSPEC_VPACK_SIGN_UNS_SAT = 239,
  UNSPEC_VPACK_UNS_UNS_SAT = 240,
  UNSPEC_VPACK_UNS_UNS_MOD = 241,
  UNSPEC_VPACK_UNS_UNS_MOD_DIRECT = 242,
  UNSPEC_VREVEV = 243,
  UNSPEC_VSLV4SI = 244,
  UNSPEC_VSLO = 245,
  UNSPEC_VSR = 246,
  UNSPEC_VSRO = 247,
  UNSPEC_VSUBCUW = 248,
  UNSPEC_VSUM4UBS = 249,
  UNSPEC_VSUM4S = 250,
  UNSPEC_VSUM2SWS = 251,
  UNSPEC_VSUMSWS = 252,
  UNSPEC_VPERM = 253,
  UNSPEC_VPERMR = 254,
  UNSPEC_VPERM_UNS = 255,
  UNSPEC_VRFIN = 256,
  UNSPEC_VCFUX = 257,
  UNSPEC_VCFSX = 258,
  UNSPEC_VCTUXS = 259,
  UNSPEC_VCTSXS = 260,
  UNSPEC_VLOGEFP = 261,
  UNSPEC_VEXPTEFP = 262,
  UNSPEC_VSLDOI = 263,
  UNSPEC_VUNPACK_HI_SIGN = 264,
  UNSPEC_VUNPACK_LO_SIGN = 265,
  UNSPEC_VUNPACK_HI_SIGN_DIRECT = 266,
  UNSPEC_VUNPACK_LO_SIGN_DIRECT = 267,
  UNSPEC_VUPKHPX = 268,
  UNSPEC_VUPKLPX = 269,
  UNSPEC_CONVERT_4F32_8I16 = 270,
  UNSPEC_CONVERT_4F32_8F16 = 271,
  UNSPEC_DST = 272,
  UNSPEC_DSTT = 273,
  UNSPEC_DSTST = 274,
  UNSPEC_DSTSTT = 275,
  UNSPEC_LVSL = 276,
  UNSPEC_LVSR = 277,
  UNSPEC_LVE = 278,
  UNSPEC_STVX = 279,
  UNSPEC_STVXL = 280,
  UNSPEC_STVE = 281,
  UNSPEC_SET_VSCR = 282,
  UNSPEC_GET_VRSAVE = 283,
  UNSPEC_LVX = 284,
  UNSPEC_REDUC_PLUS = 285,
  UNSPEC_VECSH = 286,
  UNSPEC_EXTEVEN_V4SI = 287,
  UNSPEC_EXTEVEN_V8HI = 288,
  UNSPEC_EXTEVEN_V16QI = 289,
  UNSPEC_EXTEVEN_V4SF = 290,
  UNSPEC_EXTODD_V4SI = 291,
  UNSPEC_EXTODD_V8HI = 292,
  UNSPEC_EXTODD_V16QI = 293,
  UNSPEC_EXTODD_V4SF = 294,
  UNSPEC_INTERHI_V4SI = 295,
  UNSPEC_INTERHI_V8HI = 296,
  UNSPEC_INTERHI_V16QI = 297,
  UNSPEC_INTERLO_V4SI = 298,
  UNSPEC_INTERLO_V8HI = 299,
  UNSPEC_INTERLO_V16QI = 300,
  UNSPEC_LVLX = 301,
  UNSPEC_LVLXL = 302,
  UNSPEC_LVRX = 303,
  UNSPEC_LVRXL = 304,
  UNSPEC_STVLX = 305,
  UNSPEC_STVLXL = 306,
  UNSPEC_STVRX = 307,
  UNSPEC_STVRXL = 308,
  UNSPEC_VSLV = 309,
  UNSPEC_VSRV = 310,
  UNSPEC_VMULWHUB = 311,
  UNSPEC_VMULWLUB = 312,
  UNSPEC_VMULWHSB = 313,
  UNSPEC_VMULWLSB = 314,
  UNSPEC_VMULWHUH = 315,
  UNSPEC_VMULWLUH = 316,
  UNSPEC_VMULWHSH = 317,
  UNSPEC_VMULWLSH = 318,
  UNSPEC_VUPKHU = 319,
  UNSPEC_VUPKLU = 320,
  UNSPEC_VPERMSI = 321,
  UNSPEC_VPERMHI = 322,
  UNSPEC_INTERHI = 323,
  UNSPEC_INTERLO = 324,
  UNSPEC_VUPKHS_V4SF = 325,
  UNSPEC_VUPKLS_V4SF = 326,
  UNSPEC_VUPKHU_V4SF = 327,
  UNSPEC_VUPKLU_V4SF = 328,
  UNSPEC_VGBBD = 329,
  UNSPEC_VSPLT_DIRECT = 330,
  UNSPEC_VMRGEW_DIRECT = 331,
  UNSPEC_VMRGOW_DIRECT = 332,
  UNSPEC_VSUMSWS_DIRECT = 333,
  UNSPEC_VADDCUQ = 334,
  UNSPEC_VADDEUQM = 335,
  UNSPEC_VADDECUQ = 336,
  UNSPEC_VSUBCUQ = 337,
  UNSPEC_VSUBEUQM = 338,
  UNSPEC_VSUBECUQ = 339,
  UNSPEC_VBPERMQ = 340,
  UNSPEC_VBPERMD = 341,
  UNSPEC_BCDADD = 342,
  UNSPEC_BCDSUB = 343,
  UNSPEC_BCD_OVERFLOW = 344,
  UNSPEC_BCDSHIFT = 345,
  UNSPEC_VRLMI = 346,
  UNSPEC_VRLNM = 347,
  UNSPEC_VCFUGED = 348,
  UNSPEC_VCLZDM = 349,
  UNSPEC_VCTZDM = 350,
  UNSPEC_VGNB = 351,
  UNSPEC_VPDEPD = 352,
  UNSPEC_VPEXTD = 353,
  UNSPEC_VCLRLB = 354,
  UNSPEC_VCLRRB = 355,
  UNSPEC_VSTRIR = 356,
  UNSPEC_VSTRIL = 357,
  UNSPEC_SLDB = 358,
  UNSPEC_SRDB = 359,
  UNSPEC_VECTOR_SHIFT = 360,
  UNSPEC_VSX_ASSEMBLE = 361,
  UNSPEC_MMA_EXTRACT = 362,
  UNSPEC_MMA_PMXVBF16GER2 = 363,
  UNSPEC_MMA_PMXVBF16GER2NN = 364,
  UNSPEC_MMA_PMXVBF16GER2NP = 365,
  UNSPEC_MMA_PMXVBF16GER2PN = 366,
  UNSPEC_MMA_PMXVBF16GER2PP = 367,
  UNSPEC_MMA_PMXVF16GER2 = 368,
  UNSPEC_MMA_PMXVF16GER2NN = 369,
  UNSPEC_MMA_PMXVF16GER2NP = 370,
  UNSPEC_MMA_PMXVF16GER2PN = 371,
  UNSPEC_MMA_PMXVF16GER2PP = 372,
  UNSPEC_MMA_PMXVF32GER = 373,
  UNSPEC_MMA_PMXVF32GERNN = 374,
  UNSPEC_MMA_PMXVF32GERNP = 375,
  UNSPEC_MMA_PMXVF32GERPN = 376,
  UNSPEC_MMA_PMXVF32GERPP = 377,
  UNSPEC_MMA_PMXVF64GER = 378,
  UNSPEC_MMA_PMXVF64GERNN = 379,
  UNSPEC_MMA_PMXVF64GERNP = 380,
  UNSPEC_MMA_PMXVF64GERPN = 381,
  UNSPEC_MMA_PMXVF64GERPP = 382,
  UNSPEC_MMA_PMXVI16GER2 = 383,
  UNSPEC_MMA_PMXVI16GER2PP = 384,
  UNSPEC_MMA_PMXVI16GER2S = 385,
  UNSPEC_MMA_PMXVI16GER2SPP = 386,
  UNSPEC_MMA_PMXVI4GER8 = 387,
  UNSPEC_MMA_PMXVI4GER8PP = 388,
  UNSPEC_MMA_PMXVI8GER4 = 389,
  UNSPEC_MMA_PMXVI8GER4PP = 390,
  UNSPEC_MMA_PMXVI8GER4SPP = 391,
  UNSPEC_MMA_XVBF16GER2 = 392,
  UNSPEC_MMA_XVBF16GER2NN = 393,
  UNSPEC_MMA_XVBF16GER2NP = 394,
  UNSPEC_MMA_XVBF16GER2PN = 395,
  UNSPEC_MMA_XVBF16GER2PP = 396,
  UNSPEC_MMA_XVF16GER2 = 397,
  UNSPEC_MMA_XVF16GER2NN = 398,
  UNSPEC_MMA_XVF16GER2NP = 399,
  UNSPEC_MMA_XVF16GER2PN = 400,
  UNSPEC_MMA_XVF16GER2PP = 401,
  UNSPEC_MMA_XVF32GER = 402,
  UNSPEC_MMA_XVF32GERNN = 403,
  UNSPEC_MMA_XVF32GERNP = 404,
  UNSPEC_MMA_XVF32GERPN = 405,
  UNSPEC_MMA_XVF32GERPP = 406,
  UNSPEC_MMA_XVF64GER = 407,
  UNSPEC_MMA_XVF64GERNN = 408,
  UNSPEC_MMA_XVF64GERNP = 409,
  UNSPEC_MMA_XVF64GERPN = 410,
  UNSPEC_MMA_XVF64GERPP = 411,
  UNSPEC_MMA_XVI16GER2 = 412,
  UNSPEC_MMA_XVI16GER2PP = 413,
  UNSPEC_MMA_XVI16GER2S = 414,
  UNSPEC_MMA_XVI16GER2SPP = 415,
  UNSPEC_MMA_XVI4GER8 = 416,
  UNSPEC_MMA_XVI4GER8PP = 417,
  UNSPEC_MMA_XVI8GER4 = 418,
  UNSPEC_MMA_XVI8GER4PP = 419,
  UNSPEC_MMA_XVI8GER4SPP = 420,
  UNSPEC_MMA_XXMFACC = 421,
  UNSPEC_MMA_XXMTACC = 422,
  UNSPEC_MOVSD_LOAD = 423,
  UNSPEC_MOVSD_STORE = 424,
  UNSPEC_DDEDPD = 425,
  UNSPEC_DENBCD = 426,
  UNSPEC_DXEX = 427,
  UNSPEC_DIEX = 428,
  UNSPEC_DSCLI = 429,
  UNSPEC_DTSTSFI = 430,
  UNSPEC_DSCRI = 431,
  UNSPEC_DQUAN = 432,
  UNSPEC_VCIPHER = 433,
  UNSPEC_VNCIPHER = 434,
  UNSPEC_VCIPHERLAST = 435,
  UNSPEC_VNCIPHERLAST = 436,
  UNSPEC_VSBOX = 437,
  UNSPEC_VSHASIGMA = 438,
  UNSPEC_VPERMXOR = 439,
  UNSPEC_VPMSUM = 440,
  UNSPEC_HTM_FENCE = 441,
  UNSPEC_PCREL_OPT_LD_ADDR = 442,
  UNSPEC_PCREL_OPT_LD_DATA = 443,
  UNSPEC_PCREL_OPT_LD_SAME_REG = 444,
  UNSPEC_PCREL_OPT_LD_RELOC = 445,
  UNSPEC_PCREL_OPT_ST_ADDR = 446,
  UNSPEC_PCREL_OPT_ST_RELOC = 447
};
#define NUM_UNSPEC_VALUES 448
extern const char *const unspec_strings[];

enum unspecv {
  UNSPECV_BLOCK = 0,
  UNSPECV_LL = 1,
  UNSPECV_SC = 2,
  UNSPECV_PROBE_STACK_RANGE = 3,
  UNSPECV_EH_RR = 4,
  UNSPECV_ISYNC = 5,
  UNSPECV_MFTB = 6,
  UNSPECV_DARN = 7,
  UNSPECV_NLGR = 8,
  UNSPECV_MFFS = 9,
  UNSPECV_MFFSL = 10,
  UNSPECV_MFFSCRN = 11,
  UNSPECV_MFFSCDRN = 12,
  UNSPECV_MTFSF = 13,
  UNSPECV_MTFSF_HI = 14,
  UNSPECV_MTFSB0 = 15,
  UNSPECV_MTFSB1 = 16,
  UNSPECV_SPLIT_STACK_RETURN = 17,
  UNSPECV_SPEC_BARRIER = 18,
  UNSPECV_PLT16_LO = 19,
  UNSPECV_PLT_PCREL = 20,
  UNSPECV_SET_VRSAVE = 21,
  UNSPECV_MTVSCR = 22,
  UNSPECV_MFVSCR = 23,
  UNSPECV_DSSALL = 24,
  UNSPECV_DSS = 25,
  UNSPECV_MMA_ASSEMBLE = 26,
  UNSPECV_MMA_XXSETACCZ = 27,
  UNSPECV_HTM_TABORT = 28,
  UNSPECV_HTM_TABORTXC = 29,
  UNSPECV_HTM_TABORTXCI = 30,
  UNSPECV_HTM_TBEGIN = 31,
  UNSPECV_HTM_TCHECK = 32,
  UNSPECV_HTM_TEND = 33,
  UNSPECV_HTM_TRECHKPT = 34,
  UNSPECV_HTM_TRECLAIM = 35,
  UNSPECV_HTM_TSR = 36,
  UNSPECV_HTM_TTEST = 37,
  UNSPECV_HTM_MFSPR = 38,
  UNSPECV_HTM_MTSPR = 39
};
#define NUM_UNSPECV_VALUES 40
extern const char *const unspecv_strings[];

#endif /* GCC_INSN_CONSTANTS_H */
